@@ -21,12 +21,12 @@ class ode
 A generic interface class to numeric integrators. It has the following
 methods::
 
-    integrator = ode(f,jac=None)
-    integrator = integrator.set_integrator(name,**params)
-    integrator = integrator.set_initial_value(y0,t0=0.0)
+    integrator = ode(f, jac=None)
+    integrator = integrator.set_integrator(name, **params)
+    integrator = integrator.set_initial_value(y0, t0=0.0)
     integrator = integrator.set_f_params(*args)
     integrator = integrator.set_jac_params(*args)
-    y1 = integrator.integrate(t1,step=0,relax=0)
+    y1 = integrator.integrate(t1, step=False, relax=False)
     flag = integrator.successful()
 
 class complex_ode
@@ -39,7 +39,6 @@ an alternative to ode with the zvode solver, sometimes performing better.
 """
 from __future__ import division, print_function, absolute_import
 
-
 # XXX: Integrators must have:
 # ===========================
 # cvode - C version of vode and vodpk with many improvements.
@@ -70,7 +69,7 @@ from __future__ import division, print_function, absolute_import
 #         # arguments
 #         # to these functions.
 #         <calculate y1>
-#         if <calculation was unsuccesful>:
+#         if <calculation was unsuccessful>:
 #             self.success = 0
 #         return t1,y1
 #
@@ -95,9 +94,9 @@ from . import _dop
 from . import lsoda as _lsoda
 
 
-#------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # User interface
-#------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 
 
 class ode(object):
@@ -325,16 +324,16 @@ class ode(object):
     >>> dt = 1
     >>> while r.successful() and r.t < t1:
     ...     print(r.t+dt, r.integrate(r.t+dt))
-    (1, array([-0.71038232+0.23749653j,  0.40000271+0.j        ]))
-    (2.0, array([ 0.19098503-0.52359246j,  0.22222356+0.j        ]))
-    (3.0, array([ 0.47153208+0.52701229j,  0.15384681+0.j        ]))
-    (4.0, array([-0.61905937+0.30726255j,  0.11764744+0.j        ]))
-    (5.0, array([ 0.02340997-0.61418799j,  0.09523835+0.j        ]))
-    (6.0, array([ 0.58643071+0.339819j,  0.08000018+0.j      ]))
-    (7.0, array([-0.52070105+0.44525141j,  0.06896565+0.j        ]))
-    (8.0, array([-0.15986733-0.61234476j,  0.06060616+0.j        ]))
-    (9.0, array([ 0.64850462+0.15048982j,  0.05405414+0.j        ]))
-    (10.0, array([-0.38404699+0.56382299j,  0.04878055+0.j        ]))
+    1 [-0.71038232+0.23749653j  0.40000271+0.j        ]
+    2.0 [0.19098503-0.52359246j 0.22222356+0.j        ]
+    3.0 [0.47153208+0.52701229j 0.15384681+0.j        ]
+    4.0 [-0.61905937+0.30726255j  0.11764744+0.j        ]
+    5.0 [0.02340997-0.61418799j 0.09523835+0.j        ]
+    6.0 [0.58643071+0.339819j 0.08000018+0.j      ]
+    7.0 [-0.52070105+0.44525141j  0.06896565+0.j        ]
+    8.0 [-0.15986733-0.61234476j  0.06060616+0.j        ]
+    9.0 [0.64850462+0.15048982j 0.05405414+0.j        ]
+    10.0 [-0.38404699+0.56382299j  0.04878055+0.j        ]
 
     References
     ----------
@@ -344,6 +343,7 @@ class ode(object):
         Springer-Verlag (1993)
 
     """
+
     def __init__(self, f, jac=None):
         self.stiff = 0
         self.f = f
@@ -384,7 +384,7 @@ class ode(object):
             # FIXME: this really should be raise an exception. Will that break
             # any code?
             warnings.warn('No integrator name match with %r or is not '
-                'available.' % name)
+                          'available.' % name)
         else:
             self._integrator = integrator(**integrator_params)
             if not len(self._y):
@@ -393,8 +393,32 @@ class ode(object):
             self._integrator.reset(len(self._y), self.jac is not None)
         return self
 
-    def integrate(self, t, step=0, relax=0):
-        """Find y=y(t), set y as an initial condition, and return y."""
+    def integrate(self, t, step=False, relax=False):
+        """Find y=y(t), set y as an initial condition, and return y.
+
+        Parameters
+        ----------
+        t : float
+            The endpoint of the integration step.
+        step : bool
+            If True, and if the integrator supports the step method,
+            then perform a single integration step and return.
+            This parameter is provided in order to expose internals of
+            the implementation, and should not be changed from its default
+            value in most cases.
+        relax : bool
+            If True and if the integrator supports the run_relax method,
+            then integrate until t_1 >= t and return. ``relax`` is not
+            referenced if ``step=True``.
+            This parameter is provided in order to expose internals of
+            the implementation, and should not be changed from its default
+            value in most cases.
+
+        Returns
+        -------
+        y : float
+            The integrated value at t
+        """
         if step and self._integrator.supports_step:
             mth = self._integrator.step
         elif relax and self._integrator.supports_run_relax:
@@ -404,8 +428,8 @@ class ode(object):
 
         try:
             self._y, self.t = mth(self.f, self.jac or (lambda: None),
-                                self._y, self.t, t,
-                                self.f_params, self.jac_params)
+                                  self._y, self.t, t,
+                                  self.f_params, self.jac_params)
         except SystemError:
             # f2py issue with tuple returns, see ticket 1187.
             raise ValueError('Function to integrate must not return a tuple.')
@@ -420,6 +444,97 @@ class ode(object):
             self.set_integrator('')
         return self._integrator.success == 1
 
+    def get_return_code(self):
+        """Extracts the return code for the integration to enable better control
+        if the integration fails.
+
+        In general, a return code > 0 implies success while a return code < 0
+        implies failure.
+
+        Notes
+        -----
+        This section describes possible return codes and their meaning, for available
+        integrators that can be selected by `set_integrator` method.
+
+        "vode"
+
+        ===========  =======
+        Return Code  Message
+        ===========  =======
+        2            Integration successful.
+        -1           Excess work done on this call. (Perhaps wrong MF.)
+        -2           Excess accuracy requested. (Tolerances too small.)
+        -3           Illegal input detected. (See printed message.)
+        -4           Repeated error test failures. (Check all input.)
+        -5           Repeated convergence failures. (Perhaps bad Jacobian
+                     supplied or wrong choice of MF or tolerances.)
+        -6           Error weight became zero during problem. (Solution
+                     component i vanished, and ATOL or ATOL(i) = 0.)
+        ===========  =======
+
+        "zvode"
+
+        ===========  =======
+        Return Code  Message
+        ===========  =======
+        2            Integration successful.
+        -1           Excess work done on this call. (Perhaps wrong MF.)
+        -2           Excess accuracy requested. (Tolerances too small.)
+        -3           Illegal input detected. (See printed message.)
+        -4           Repeated error test failures. (Check all input.)
+        -5           Repeated convergence failures. (Perhaps bad Jacobian
+                     supplied or wrong choice of MF or tolerances.)
+        -6           Error weight became zero during problem. (Solution
+                     component i vanished, and ATOL or ATOL(i) = 0.)
+        ===========  =======
+
+        "dopri5"
+
+        ===========  =======
+        Return Code  Message
+        ===========  =======
+        1            Integration successful.
+        2            Integration successful (interrupted by solout).
+        -1           Input is not consistent.
+        -2           Larger nsteps is needed.
+        -3           Step size becomes too small.
+        -4           Problem is probably stiff (interrupted).
+        ===========  =======
+
+        "dop853"
+
+        ===========  =======
+        Return Code  Message
+        ===========  =======
+        1            Integration successful.
+        2            Integration successful (interrupted by solout).
+        -1           Input is not consistent.
+        -2           Larger nsteps is needed.
+        -3           Step size becomes too small.
+        -4           Problem is probably stiff (interrupted).
+        ===========  =======
+
+        "lsoda"
+
+        ===========  =======
+        Return Code  Message
+        ===========  =======
+        2            Integration successful.
+        -1           Excess work done on this call (perhaps wrong Dfun type).
+        -2           Excess accuracy requested (tolerances too small).
+        -3           Illegal input detected (internal error).
+        -4           Repeated error test failures (internal error).
+        -5           Repeated convergence failures (perhaps bad Jacobian or tolerances).
+        -6           Error weight became zero during problem.
+        -7           Internal workspace insufficient to finish (internal error).
+        ===========  =======
+        """
+        try:
+            self._integrator
+        except AttributeError:
+            self.set_integrator('')
+        return self._integrator.istate
+
     def set_f_params(self, *args):
         """Set extra parameters for user-supplied function f."""
         self.f_params = args
@@ -450,7 +565,7 @@ class ode(object):
                 self._integrator.reset(len(self._y), self.jac is not None)
         else:
             raise ValueError("selected integrator does not support solout,"
-                            + " choose another one")
+                             " choose another one")
 
 
 def _transform_banded_jac(bjac):
@@ -467,7 +582,7 @@ def _transform_banded_jac(bjac):
     """
     # Shift every other column.
     newjac = zeros((bjac.shape[0] + 1, bjac.shape[1]))
-    newjac[1:,::2] = bjac[:, ::2]
+    newjac[1:, ::2] = bjac[:, ::2]
     newjac[:-1, 1::2] = bjac[:, 1::2]
     return newjac
 
@@ -504,10 +619,10 @@ class complex_ode(ode):
     def __init__(self, f, jac=None):
         self.cf = f
         self.cjac = jac
-        if jac is not None:
-            ode.__init__(self, self._wrap, self._wrap_jac)
-        else:
+        if jac is None:
             ode.__init__(self, self._wrap, None)
+        else:
+            ode.__init__(self, self._wrap, self._wrap_jac)
 
     def _wrap(self, t, y, *f_args):
         f = self.cf(*((t, y[::2] + 1j * y[1::2]) + f_args))
@@ -525,7 +640,7 @@ class complex_ode(ode):
         # entry in jac, say 2+3j, becomes a 2x2 block of the form
         #     [2 -3]
         #     [3  2]
-        jac_tmp = zeros((2*jac.shape[0], 2*jac.shape[1]))
+        jac_tmp = zeros((2 * jac.shape[0], 2 * jac.shape[1]))
         jac_tmp[1::2, 1::2] = jac_tmp[::2, ::2] = real(jac)
         jac_tmp[1::2, ::2] = imag(jac)
         jac_tmp[::2, 1::2] = -jac_tmp[1::2, ::2]
@@ -565,8 +680,8 @@ class complex_ode(ode):
             # (which are for the complex Jacobian) with the bandwidths of
             # the corresponding real-valued Jacobian wrapper of the complex
             # Jacobian.
-            integrator_params['lband'] = 2*(lband or 0) + 1
-            integrator_params['uband'] = 2*(uband or 0) + 1
+            integrator_params['lband'] = 2 * (lband or 0) + 1
+            integrator_params['uband'] = 2 * (uband or 0) + 1
 
         return ode.set_integrator(self, name, **integrator_params)
 
@@ -578,8 +693,32 @@ class complex_ode(ode):
         self.tmp[1::2] = imag(y)
         return ode.set_initial_value(self, self.tmp, t)
 
-    def integrate(self, t, step=0, relax=0):
-        """Find y=y(t), set y as an initial condition, and return y."""
+    def integrate(self, t, step=False, relax=False):
+        """Find y=y(t), set y as an initial condition, and return y.
+
+        Parameters
+        ----------
+        t : float
+            The endpoint of the integration step.
+        step : bool
+            If True, and if the integrator supports the step method,
+            then perform a single integration step and return.
+            This parameter is provided in order to expose internals of
+            the implementation, and should not be changed from its default
+            value in most cases.
+        relax : bool
+            If True and if the integrator supports the run_relax method,
+            then integrate until t_1 >= t and return. ``relax`` is not
+            referenced if ``step=True``.
+            This parameter is provided in order to expose internals of
+            the implementation, and should not be changed from its default
+            value in most cases.
+
+        Returns
+        -------
+        y : float
+            The integrated value at t
+        """
         y = ode.integrate(self, t, step, relax)
         return y[::2] + 1j * y[1::2]
 
@@ -604,9 +743,9 @@ class complex_ode(ode):
                             + "choose another one")
 
 
-#------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # ODE integrators
-#------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 
 def find_integrator(name):
     for cl in IntegratorBase.integrator_classes:
@@ -621,6 +760,7 @@ class IntegratorConcurrencyError(RuntimeError):
     only for a single problem at a time.
 
     """
+
     def __init__(self, name):
         msg = ("Integrator `%s` can be used to solve only a single problem "
                "at a time. If you want to integrate multiple problems, "
@@ -630,9 +770,9 @@ class IntegratorConcurrencyError(RuntimeError):
 
 
 class IntegratorBase(object):
-
-    runner = None            # runner is None => integrator is not available
-    success = None           # success==1 if integrator was called successfully
+    runner = None  # runner is None => integrator is not available
+    success = None  # success==1 if integrator was called successfully
+    istate = None  # istate > 0 means success, istate < 0 means failure
     supports_run_relax = None
     supports_step = None
     supports_solout = False
@@ -662,19 +802,19 @@ class IntegratorBase(object):
         defines the stoppage coordinate of the result.
         """
         raise NotImplementedError('all integrators must define '
-                'run(f, jac, t0, t1, y0, f_params, jac_params)')
+                                  'run(f, jac, t0, t1, y0, f_params, jac_params)')
 
     def step(self, f, jac, y0, t0, t1, f_params, jac_params):
         """Make one integration step and return (y1,t1)."""
         raise NotImplementedError('%s does not support step() method' %
-                self.__class__.__name__)
+                                  self.__class__.__name__)
 
     def run_relax(self, f, jac, y0, t0, t1, f_params, jac_params):
         """Integrate from t=t0 to t>=t1 and return (y1,t)."""
         raise NotImplementedError('%s does not support run_relax() method' %
-                self.__class__.__name__)
+                                  self.__class__.__name__)
 
-    #XXX: __str__ method for getting visual state of the integrator
+    # XXX: __str__ method for getting visual state of the integrator
 
 
 def _vode_banded_jac_wrapper(jacfunc, ml, jac_params):
@@ -692,7 +832,6 @@ def _vode_banded_jac_wrapper(jacfunc, ml, jac_params):
 
 
 class vode(IntegratorBase):
-
     runner = getattr(_vode, 'dvode', None)
 
     messages = {-1: 'Excess work done on this call. (Perhaps wrong MF.)',
@@ -700,9 +839,9 @@ class vode(IntegratorBase):
                 -3: 'Illegal input detected. (See printed message.)',
                 -4: 'Repeated error test failures. (Check all input.)',
                 -5: 'Repeated convergence failures. (Perhaps bad'
-                ' Jacobian supplied or wrong choice of MF or tolerances.)',
+                    ' Jacobian supplied or wrong choice of MF or tolerances.)',
                 -6: 'Error weight became zero during problem. (Solution'
-                ' component i vanished, and ATOL or ATOL(i) = 0.)'
+                    ' component i vanished, and ATOL or ATOL(i) = 0.)'
                 }
     supports_run_relax = 1
     supports_step = 1
@@ -839,7 +978,7 @@ class vode(IntegratorBase):
             iwork[1] = self.mu
         iwork[4] = self.order
         iwork[5] = self.nsteps
-        iwork[6] = 2           # mxhnil
+        iwork[6] = 2  # mxhnil
         self.iwork = iwork
 
         self.call_args = [self.rtol, self.atol, 1, 1,
@@ -863,13 +1002,15 @@ class vode(IntegratorBase):
         args = ((f, jac, y0, t0, t1) + tuple(self.call_args) +
                 (f_params, jac_params))
         y1, t, istate = self.runner(*args)
+        self.istate = istate
         if istate < 0:
-            warnings.warn(self.__class__.__name__ + ': ' +
-                          self.messages.get(istate,
-                                            'Unexpected istate=%s' % istate))
+            unexpected_istate_msg = 'Unexpected istate={:d}'.format(istate)
+            warnings.warn('{:s}: {:s}'.format(self.__class__.__name__,
+                          self.messages.get(istate, unexpected_istate_msg)))
             self.success = 0
         else:
             self.call_args[3] = 2  # upgrade istate from 1 to 2
+            self.istate = 2
         return y1, t
 
     def step(self, *args):
@@ -950,7 +1091,7 @@ class zvode(vode):
             iwork[1] = self.mu
         iwork[4] = self.order
         iwork[5] = self.nsteps
-        iwork[6] = 2           # mxhnil
+        iwork[6] = 2  # mxhnil
         self.iwork = iwork
 
         self.call_args = [self.rtol, self.atol, 1, 1,
@@ -964,7 +1105,6 @@ if zvode.runner is not None:
 
 
 class dopri5(IntegratorBase):
-
     runner = getattr(_dop, 'dopri5', None)
     name = 'dopri5'
     supports_solout = True
@@ -972,7 +1112,7 @@ class dopri5(IntegratorBase):
     messages = {1: 'computation successful',
                 2: 'comput. successful (interrupted by solout)',
                 -1: 'input is not consistent',
-                -2: 'larger nmax is needed',
+                -2: 'larger nsteps is needed',
                 -3: 'step size becomes too small',
                 -4: 'problem is probably stiff (interrupted)',
                 }
@@ -1028,11 +1168,13 @@ class dopri5(IntegratorBase):
         self.success = 1
 
     def run(self, f, jac, y0, t0, t1, f_params, jac_params):
-        x, y, iwork, idid = self.runner(*((f, t0, y0, t1) +
+        x, y, iwork, istate = self.runner(*((f, t0, y0, t1) +
                                           tuple(self.call_args) + (f_params,)))
-        if idid < 0:
-            warnings.warn(self.name + ': ' +
-                self.messages.get(idid, 'Unexpected idid=%s' % idid))
+        self.istate = istate
+        if istate < 0:
+            unexpected_istate_msg = 'Unexpected istate={:d}'.format(istate)
+            warnings.warn('{:s}: {:s}'.format(self.__class__.__name__,
+                          self.messages.get(istate, unexpected_istate_msg)))
             self.success = 0
         return y, x
 
@@ -1044,12 +1186,12 @@ class dopri5(IntegratorBase):
         else:
             return 1
 
+
 if dopri5.runner is not None:
     IntegratorBase.integrator_classes.append(dopri5)
 
 
 class dop853(dopri5):
-
     runner = getattr(_dop, 'dop853', None)
     name = 'dop853'
 
@@ -1065,18 +1207,10 @@ class dop853(dopri5):
                  method=None,
                  verbosity=-1,  # no messages if negative
                  ):
-        self.rtol = rtol
-        self.atol = atol
-        self.nsteps = nsteps
-        self.max_step = max_step
-        self.first_step = first_step
-        self.safety = safety
-        self.ifactor = ifactor
-        self.dfactor = dfactor
-        self.beta = beta
-        self.verbosity = verbosity
-        self.success = 1
-        self.set_solout(None)
+        super(self.__class__, self).__init__(rtol, atol, nsteps, max_step,
+                                             first_step, safety, ifactor,
+                                             dfactor, beta, method,
+                                             verbosity)
 
     def reset(self, n, has_jac):
         work = zeros((11 * n + 21,), float)
@@ -1095,12 +1229,12 @@ class dop853(dopri5):
                           self.iout, self.work, self.iwork]
         self.success = 1
 
+
 if dop853.runner is not None:
     IntegratorBase.integrator_classes.append(dop853)
 
 
 class lsoda(IntegratorBase):
-
     runner = getattr(_lsoda, 'lsoda', None)
     active_global_handle = 0
 
@@ -1198,7 +1332,7 @@ class lsoda(IntegratorBase):
         self.success = 1
         self.initialized = False
 
-    def run(self, f,jac,y0,t0,t1,f_params,jac_params):
+    def run(self, f, jac, y0, t0, t1, f_params, jac_params):
         if self.initialized:
             self.check_handle()
         else:
@@ -1207,13 +1341,15 @@ class lsoda(IntegratorBase):
         args = [f, y0, t0, t1] + self.call_args[:-1] + \
                [jac, self.call_args[-1], f_params, 0, jac_params]
         y1, t, istate = self.runner(*args)
+        self.istate = istate
         if istate < 0:
-            warnings.warn('lsoda: ' +
-                          self.messages.get(istate,
-                                            'Unexpected istate=%s' % istate))
+            unexpected_istate_msg = 'Unexpected istate={:d}'.format(istate)
+            warnings.warn('{:s}: {:s}'.format(self.__class__.__name__,
+                          self.messages.get(istate, unexpected_istate_msg)))
             self.success = 0
         else:
             self.call_args[3] = 2  # upgrade istate from 1 to 2
+            self.istate = 2
         return y1, t
 
     def step(self, *args):
@@ -1230,5 +1366,6 @@ class lsoda(IntegratorBase):
         self.call_args[2] = itask
         return r
 
+
 if lsoda.runner:
     IntegratorBase.integrator_classes.append(lsoda)
