@@ -6,11 +6,11 @@ import numpy as np
 from scipy.special import factorial
 
 from scipy._lib.six import xrange
+from scipy._lib._util import _asarray_validated
 
 
 __all__ = ["KroghInterpolator", "krogh_interpolate", "BarycentricInterpolator",
-           "barycentric_interpolate", "PiecewisePolynomial",
-           "piecewise_polynomial_interpolate", "approximate_taylor_polynomial"]
+           "barycentric_interpolate", "approximate_taylor_polynomial"]
 
 
 def _isscalar(x):
@@ -87,10 +87,7 @@ class _Interpolator1D(object):
 
     def _prepare_x(self, x):
         """Reshape input x array to 1-D"""
-        x = np.asarray(x)
-        if not np.issubdtype(x.dtype, np.inexact):
-            # Cast integers etc to floats
-            x = x.astype(float)
+        x = _asarray_validated(x, check_finite=False, as_inexact=True)
         x_shape = x.shape
         return x.ravel(), x_shape
 
@@ -168,6 +165,7 @@ class _Interpolator1DWithDerivatives(_Interpolator1D):
 
         Examples
         --------
+        >>> from scipy.interpolate import KroghInterpolator
         >>> KroghInterpolator([0,0,0],[1,2,3]).derivatives(0)
         array([1.0,2.0,3.0])
         >>> KroghInterpolator([0,0,0],[1,2,3]).derivatives([0,0])
@@ -267,6 +265,7 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
     To produce a polynomial that is zero at 0 and 1 and has
     derivative 2 at 0, call
 
+    >>> from scipy.interpolate import KroghInterpolator
     >>> KroghInterpolator([0,0,1],[0,2,0])
 
     This constructs the quadratic 2*X**2-2*X. The derivative condition
@@ -276,6 +275,8 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
     For another example, given xi, yi, and a derivative ypi for each
     point, appropriate arrays can be constructed as:
 
+    >>> xi = np.linspace(0, 1, 5)
+    >>> yi, ypi = np.random.rand(2, 5)
     >>> xi_k, yi_k = np.repeat(xi, 2), np.ravel(np.dstack((yi,ypi)))
     >>> KroghInterpolator(xi_k, yi_k)
 
@@ -333,24 +334,24 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
         pi = np.zeros((n, len(x)))
         w = np.zeros((n, len(x)))
         pi[0] = 1
-        p = np.zeros((len(x), self.r))
-        p += self.c[0,np.newaxis,:]
+        p = np.zeros((len(x), self.r), dtype=self.dtype)
+        p += self.c[0, np.newaxis, :]
 
-        for k in xrange(1,n):
+        for k in xrange(1, n):
             w[k-1] = x - self.xi[k-1]
-            pi[k] = w[k-1]*pi[k-1]
-            p += pi[k,:,np.newaxis]*self.c[k]
+            pi[k] = w[k-1] * pi[k-1]
+            p += pi[k, :, np.newaxis] * self.c[k]
 
-        cn = np.zeros((max(der,n+1), len(x), r), dtype=self.dtype)
-        cn[:n+1,:,:] += self.c[:n+1,np.newaxis,:]
+        cn = np.zeros((max(der, n+1), len(x), r), dtype=self.dtype)
+        cn[:n+1, :, :] += self.c[:n+1, np.newaxis, :]
         cn[0] = p
-        for k in xrange(1,n):
-            for i in xrange(1,n-k+1):
-                pi[i] = w[k+i-1]*pi[i-1]+pi[i]
-                cn[k] = cn[k]+pi[i,:,np.newaxis]*cn[k+i]
+        for k in xrange(1, n):
+            for i in xrange(1, n-k+1):
+                pi[i] = w[k+i-1]*pi[i-1] + pi[i]
+                cn[k] = cn[k] + pi[i, :, np.newaxis]*cn[k+i]
             cn[k] *= factorial(k)
 
-        cn[n,:,:] = 0
+        cn[n, :, :] = 0
         return cn[:der]
 
 
@@ -663,269 +664,3 @@ def barycentric_interpolate(xi, yi, x, axis=0):
 
     """
     return BarycentricInterpolator(xi, yi, axis=axis)(x)
-
-
-class PiecewisePolynomial(_Interpolator1DWithDerivatives):
-    """Piecewise polynomial curve specified by points and derivatives
-
-    This class represents a curve that is a piecewise polynomial. It
-    passes through a list of points and has specified derivatives at
-    each point. The degree of the polynomial may vary from segment to
-    segment, as may the number of derivatives available. The degree
-    should not exceed about thirty.
-
-    Appending points to the end of the curve is efficient.
-
-    Parameters
-    ----------
-    xi : array_like
-        A sorted 1-d array of x-coordinates.
-    yi : array_like or list of array_likes
-        ``yi[i][j]`` is the j-th derivative known at ``xi[i]`` (for ``axis=0``).
-    orders : list of int, or int, optional
-        A list of polynomial orders, or a single universal order.
-    direction : {None, 1, -1}, optional
-        Indicates whether the `xi` are increasing or decreasing:
-
-          +1 : increasing values
-
-          -1 : decreasing values
-
-          None : direction will be deduced from the first two elements of `xi`
-
-    axis : int, optional
-        Axis in the `yi` array corresponding to the x-coordinate values.
-
-    Notes
-    -----
-    If orders is None, or ``orders[i]`` is None, then the degree of the
-    polynomial segment is exactly the degree required to match all ``i``
-    available derivatives at both endpoints.  If ``orders[i]`` is not None,
-    then some derivatives will be ignored. The code will try to use an
-    equal number of derivatives from each end; if the total number of
-    derivatives needed is odd, it will prefer the rightmost endpoint. If
-    not enough derivatives are available, an exception is raised.
-
-    """
-    def __init__(self, xi, yi, orders=None, direction=None, axis=0):
-        _Interpolator1DWithDerivatives.__init__(self, axis=axis)
-
-        warnings.warn('PiecewisePolynomial is deprecated in scipy 0.14. '
-                      'Use BPoly.from_derivatives instead.',
-                category=DeprecationWarning)
-
-        if axis != 0:
-            try:
-                yi = np.asarray(yi)
-            except ValueError:
-                raise ValueError("If yi is a list, then axis must be 0")
-
-            preslice = ((slice(None,None,None),) * (axis % yi.ndim))
-            slice0 = preslice + (0,)
-            slice1 = preslice + (slice(1, None, None),)
-        else:
-            slice0 = 0
-            slice1 = slice(1, None, None)
-
-        yi0 = np.asarray(yi[slice0])
-        self._set_yi(yi0)
-
-        self.xi = [xi[0]]
-        self.yi = [self._reshape_yi(yi0)]
-        self.n = 1
-        self.r = np.prod(self._y_extra_shape, dtype=np.int64)
-
-        self.direction = direction
-        self.orders = []
-        self.polynomials = []
-        self.extend(xi[1:],yi[slice1],orders)
-
-    def _make_polynomial(self,x1,y1,x2,y2,order,direction):
-        """Construct the interpolating polynomial object
-
-        Deduces the number of derivatives to match at each end
-        from order and the number of derivatives available. If
-        possible it uses the same number of derivatives from
-        each end; if the number is odd it tries to take the
-        extra one from y2. In any case if not enough derivatives
-        are available at one end or another it draws enough to
-        make up the total from the other end.
-        """
-        n = order+1
-        n1 = min(n//2,len(y1))
-        n2 = min(n-n1,len(y2))
-        n1 = min(n-n2,len(y1))
-        if n1+n2 != n:
-            raise ValueError("Point %g has %d derivatives, point %g has %d "
-                             "derivatives, but order %d requested" % (x1,
-                             len(y1), x2, len(y2), order))
-
-        if not (n1 <= len(y1) and n2 <= len(y2)):
-            raise ValueError("`order` input incompatible with length y1 or y2.")
-
-        xi = np.zeros(n)
-        yi = np.zeros((n, self.r), dtype=self.dtype)
-
-        xi[:n1] = x1
-        yi[:n1] = y1[:n1].reshape((n1, self.r))
-        xi[n1:] = x2
-        yi[n1:] = y2[:n2].reshape((n2, self.r))
-
-        return KroghInterpolator(xi,yi,axis=0)
-
-    def append(self, xi, yi, order=None):
-        """
-        Append a single point with derivatives to the PiecewisePolynomial
-
-        Parameters
-        ----------
-        xi : float
-            Point to add.
-        yi : array_like
-            `yi` is the list of derivatives known at `xi`.
-        order : int or None, optional
-            A polynomial order, or instructions to use the highest possible
-            order.
-
-        """
-        yi = self._reshape_yi(yi, check=True)
-        self._set_dtype(yi.dtype, union=True)
-
-        if self.direction is None:
-            self.direction = np.sign(xi-self.xi[-1])
-        elif (xi-self.xi[-1])*self.direction < 0:
-            raise ValueError("x coordinates must be in the %d direction: %s" % (
-                             self.direction, self.xi))
-
-        self.xi.append(xi)
-        self.yi.append(yi)
-
-        if order is None:
-            n1 = len(self.yi[-2])
-            n2 = len(self.yi[-1])
-            n = n1+n2
-            order = n-1
-
-        self.orders.append(order)
-        self.polynomials.append(self._make_polynomial(
-            self.xi[-2], self.yi[-2],
-            self.xi[-1], self.yi[-1],
-            order, self.direction))
-        self.n += 1
-
-    def extend(self, xi, yi, orders=None):
-        """
-        Extend the PiecewisePolynomial by a list of points
-
-        Parameters
-        ----------
-        xi : array_like
-            A sorted list of x-coordinates.
-        yi : list of lists of length N1
-            ``yi[i]`` (if ``axis == 0``) is the list of derivatives known
-            at ``xi[i]``.
-        orders : int or list of ints, optional
-            A list of polynomial orders, or a single universal order.
-
-        """
-        if self._y_axis == 0:
-            # allow yi to be a ragged list
-            for i in xrange(len(xi)):
-                if orders is None or _isscalar(orders):
-                    self.append(xi[i],yi[i],orders)
-                else:
-                    self.append(xi[i],yi[i],orders[i])
-        else:
-            preslice = (slice(None,None,None),) * self._y_axis
-            for i in xrange(len(xi)):
-                if orders is None or _isscalar(orders):
-                    self.append(xi[i],yi[preslice + (i,)],orders)
-                else:
-                    self.append(xi[i],yi[preslice + (i,)],orders[i])
-
-    def _evaluate(self, x):
-        if _isscalar(x):
-            pos = np.clip(np.searchsorted(self.xi, x) - 1, 0, self.n-2)
-            y = self.polynomials[pos](x)
-        else:
-            m = len(x)
-            pos = np.clip(np.searchsorted(self.xi, x) - 1, 0, self.n-2)
-            y = np.zeros((m, self.r), dtype=self.dtype)
-            if y.size > 0:
-                for i in xrange(self.n-1):
-                    c = pos == i
-                    y[c] = self.polynomials[i](x[c])
-        return y
-
-    def _evaluate_derivatives(self, x, der=None):
-        if der is None and self.polynomials:
-            der = self.polynomials[0].n
-        if _isscalar(x):
-            pos = np.clip(np.searchsorted(self.xi, x) - 1, 0, self.n-2)
-            y = self.polynomials[pos].derivatives(x,der=der)
-        else:
-            m = len(x)
-            pos = np.clip(np.searchsorted(self.xi, x) - 1, 0, self.n-2)
-            y = np.zeros((der,m,self.r), dtype=self.dtype)
-            if y.size > 0:
-                for i in xrange(self.n-1):
-                    c = pos == i
-                    y[:,c] = self.polynomials[i].derivatives(x[c],der=der)
-        return y
-
-
-def piecewise_polynomial_interpolate(xi, yi, x, orders=None, der=0, axis=0):
-    """
-    Convenience function for piecewise polynomial interpolation.
-
-    Parameters
-    ----------
-    xi : array_like
-        A sorted list of x-coordinates.
-    yi : list of lists
-        ``yi[i]`` is the list of derivatives known at ``xi[i]``.
-    x : scalar or array_like
-        Coordinates at which to evalualte the polynomial.
-    orders : int or list of ints, optional
-        A list of polynomial orders, or a single universal order.
-    der : int or list, optional
-        How many derivatives to extract; None for all potentially
-        nonzero derivatives (that is a number equal to the number
-        of points), or a list of derivatives to extract. This number
-        includes the function value as 0th derivative.
-    axis : int, optional
-        Axis in the `yi` array corresponding to the x-coordinate values.
-
-    Returns
-    -------
-    y : ndarray
-        Interpolated values or derivatives. If multiple derivatives
-        were requested, these are given along the first axis.
-
-    See Also
-    --------
-    PiecewisePolynomial
-
-    Notes
-    -----
-    If `orders` is None, or ``orders[i]`` is None, then the degree of the
-    polynomial segment is exactly the degree required to match all i
-    available derivatives at both endpoints.  If ``orders[i]`` is not None,
-    then some derivatives will be ignored.  The code will try to use an
-    equal number of derivatives from each end; if the total number of
-    derivatives needed is odd, it will prefer the rightmost endpoint. If
-    not enough derivatives are available, an exception is raised.
-
-    Construction of these piecewise polynomials can be an expensive process;
-    if you repeatedly evaluate the same polynomial, consider using the class
-    PiecewisePolynomial (which is what this function does).
-
-    """
-
-    P = PiecewisePolynomial(xi, yi, orders, axis=axis)
-    if der == 0:
-        return P(x)
-    elif _isscalar(der):
-        return P.derivative(x,der=der)
-    else:
-        return P.derivatives(x,der=np.amax(der)+1)[der]
