@@ -1,8 +1,8 @@
 from __future__ import division, print_function, absolute_import
-from scipy._lib.six import with_metaclass
-from numpy.testing import dec
+import numpy as np
+import pytest
 
-from scipy.special._testutils import MissingModule, check_version, DecoratorMeta
+from scipy.special._testutils import MissingModule, check_version
 from scipy.special._mptestutils import mp_assert_allclose
 from scipy.special._precompute.utils import lagrange_inversion
 
@@ -14,17 +14,17 @@ except ImportError:
 try:
     import mpmath as mp
 except ImportError:
-    try:
-        from sympy import mpmath as mp
-    except ImportError:
-        mp = MissingModule('mpmath')
+    mp = MissingModule('mpmath')
 
 
-class TestInversion(with_metaclass(DecoratorMeta, object)):
-    decorators = [(dec.slow, None),
-                  (check_version, (sympy, '0.7')),
-                  (check_version, (mp, '0.19'))]
+_is_32bit_platform = np.intp(0).itemsize < 8
 
+
+@pytest.mark.slow
+@check_version(sympy, '0.7')
+@check_version(mp, '0.19')
+class TestInversion(object):
+    @pytest.mark.xfail(condition=_is_32bit_platform, reason="rtol only 2e-9, see gh-6938")
     def test_log(self):
         with mp.workdps(30):
             logcoeffs = mp.taylor(lambda x: mp.log(1 + x), 0, 10)
@@ -32,9 +32,11 @@ class TestInversion(with_metaclass(DecoratorMeta, object)):
             invlogcoeffs = lagrange_inversion(logcoeffs)
             mp_assert_allclose(invlogcoeffs, expcoeffs)
 
+    @pytest.mark.xfail(condition=_is_32bit_platform, reason="rtol only 1e-15, see gh-6938")
     def test_sin(self):
         with mp.workdps(30):
             sincoeffs = mp.taylor(mp.sin, 0, 10)
             asincoeffs = mp.taylor(mp.asin, 0, 10)
             invsincoeffs = lagrange_inversion(sincoeffs)
             mp_assert_allclose(invsincoeffs, asincoeffs, atol=1e-30)
+
